@@ -247,9 +247,7 @@ fn main() {
   }
   let numbers = staticvec![1, 2, 3, 4, 5];
   let zero = "0".to_string();
-  let result = numbers
-    .iter()
-    .rfold(zero, |acc, &x| format!("({} + {})", x, acc));
+  let result = numbers.iter().rfold(zero, |acc, &x| format!("({} + {})", x, acc));
   println!("{}", result);
   let mut strings = staticvec!["foo", "bar", "baz", "qux"];
   println!("{}", strings.swap_remove(1));
@@ -306,4 +304,20 @@ fn main() {
   for i in &eight_from {
     println!("{}", i);
   }
+  for i in &staticvec![1, 2, 3, 4, 5, 6, 7, 8].split_off(3) {
+    println!("{}", i);
+  }
+  let mut vec1 = staticvec!["foo", "bar", "Bar", "baz", "bar"];
+  vec1.dedup_by(|a, b| a.eq_ignore_ascii_case(b));
+  assert_eq!(vec1.as_slice(), ["foo", "bar", "baz", "bar"]);
+  let mut vec2 = staticvec![1, 2, 2, 3, 2];
+  vec2.dedup();
+  assert_eq!(vec2.as_slice(), [1, 2, 3, 2]);
+  let mut vec3 = staticvec![10, 20, 21, 30, 20];
+  vec3.dedup_by_key(|i| *i / 10);
+  assert_eq!(vec3.as_slice(), [10, 20, 30, 20]);
+  let mut vec4 = vec![1, 2, 3];
+  let vec5 = vec4.split_off(1);
+  assert_eq!(vec4.as_slice(), [1]);
+  assert_eq!(vec5.as_slice(), [2, 3]);
 }
